@@ -32,6 +32,7 @@ _SECTION_HEADER_FORMAT = (
 )
 _SECTION_HEADER_SIZE = struct.calcsize(_SECTION_HEADER_FORMAT)
 _NEW_SECTION_CHARACTERISTICS = pefile.SECTION_CHARACTERISTICS['IMAGE_SCN_MEM_READ'] | pefile.SECTION_CHARACTERISTICS['IMAGE_SCN_CNT_INITIALIZED_DATA']
+_ATTRIBUTE_CERTIFICATE_TABLE_ALIGNMENT = 8
 
 
 class PEContext:
@@ -215,7 +216,8 @@ def find_library(name: str, wheel_dirs: typing.Optional[typing.Iterable], arch:
     given architecture.
 
     1. If not None, the directories in wheel_dirs.
-    2. The PATH environment variable. (If we are on a case-sensitive file
+    2. The PATH environment variable, with any applicable adjustments due to
+       the Windows file system redirector. (If we are on a case-sensitive file
        system and a directory contains more than one DLL with the correct
        architecture that differs by case only, then choose one arbitrarily.)"""
     name = name.lower()
@@ -320,7 +322,7 @@ def get_all_needed(lib_path: str,
     DLL names of all dependent DLLs that cannot be found.
 
     no_dlls is a set of DLL names to force exclusion from the wheel. We do not
-    search for dependencies of these DLLs. Cannot overlap with add_dlls.
+    search for dependencies of these DLLs.
 
     If wheel_dirs is not None, it is an iterable of directories in the wheel
     where dependencies are searched first."""
@@ -389,6 +391,10 @@ def _get_pe_size_and_enough_padding(pe: pefile.PE, new_dlls: typing.Iterable[byt
     algorithm. new_dlls must be in a deterministic order in order for the Next
     Fit algorithm to return a deterministic result.
 
+    Side effect: pe.sections is sorted by VirtualAddress. In most cases this
+    should make no difference because the PE specification requires that the
+    sections be ordered by VirtualAddress.
+
     Precondition: pe must have at least 1 DLL dependency"""
     pe_size = 0
     new_dlls = list(new_dlls)
@@ -413,13 +419,26 @@ def replace_needed(lib_path: str, old_deps: typing.List[str], name_map: typing.D
     name_map: a dict that maps an old dependency name to a new name, must
         contain at least all the keys in old_deps
     strip: whether to try to strip DLLs that contain trailing data if not
-        enough internal padding exists"""
+        enough internal padding exists
+    verbose: verbosity level, 0 to 2
+    test: testing options for internal use"""
     if not old_deps:
         # no dependency names to change
         return
     name_map = {dep.lower().encode('utf-8'): name_map[dep].encode('utf-8') for dep in old_deps}
         # keep only the DLLs that will be mangled
 
+    # If an attribute certificate table exists and is the only trailing data,
+    # remove the table. In this case, we end up removing all trailing data
+    # without needing to run strip.
+    with PEContext(lib_path, None, False, verbose) as pe:
+        pe_size = max(section.PointerToRawData + section.SizeOfRawData for section in pe.sections)
+        cert_table = pe.OPTIONAL_HEADER.DATA_DIRECTORY[pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_SECURITY']]
+        truncate = cert_table.VirtualAddress == _round_to_next(pe_size, _ATTRIBUTE_CERTIFICATE_TABLE_ALIGNMENT) and cert_table.VirtualAddress + cert_table.Size == os.path.getsize(lib_path)
+    if truncate:
+        with open(lib_path, 'rb+') as f:
+            f.truncate(pe_size)
+
     # New dependency names are longer than the old ones, so we cannot simply
     # overwrite the bytes of the old dependency names. Determine whether the PE
     # file has enough usable internal padding to use to write the new
@@ -444,10 +463,7 @@ def replace_needed(lib_path: str, old_deps: typing.List[str], name_map: typing.D
             pe_size, enough_padding = _get_pe_size_and_enough_padding(pe, name_map.values())
 
     lib_name = os.path.basename(lib_path)
-    with open(lib_path, 'rb') as f:
-        # workaround for https://github.com/erocarrera/pefile/issues/356
-        lib_data = f.read()
-    if not enough_padding and pe_size < len(lib_data):
+    if not enough_padding and pe_size < os.path.getsize(lib_path):
         # cannot rename dependencies due to trailing data
         if strip:
             raise RuntimeError(textwrap.fill(
@@ -495,6 +511,9 @@ def replace_needed(lib_path: str, old_deps: typing.List[str], name_map: typing.D
             ]
             raise RuntimeError(''.join(error_text))
 
+    with open(lib_path, 'rb') as f:
+        # workaround for https://github.com/erocarrera/pefile/issues/356
+        lib_data = f.read()
     with PEContext(None, lib_data, True, verbose) as pe:
         if enough_padding:
             # overwrite padding with new DLL names
@@ -549,10 +568,7 @@ def replace_needed(lib_path: str, old_deps: typing.List[str], name_map: typing.D
 
             # update PE headers to what they will need to be once the
             # new section header and new section are added
-            section_table_end = (pe.DOS_HEADER.e_lfanew + 4 +
-                                 pe.FILE_HEADER.sizeof() +
-                                 pe.FILE_HEADER.SizeOfOptionalHeader +
-                                 pe.FILE_HEADER.NumberOfSections * _SECTION_HEADER_SIZE)
+            section_table_end = pe.sections[-1].get_file_offset() + _SECTION_HEADER_SIZE
             if 'header_space' not in test and pe.OPTIONAL_HEADER.SizeOfHeaders - section_table_end >= _SECTION_HEADER_SIZE:
                 # there's enough unused space to add new section header
                 new_section_header_space_needed = 0
@@ -584,12 +600,10 @@ def replace_needed(lib_path: str, old_deps: typing.List[str], name_map: typing.D
                     if old_dll in new_section_offset_mapping:
                         entry.struct.szName = new_section_rva + new_section_offset_mapping[old_dll]
 
-        # clear any signatures
+        # clear reference to attribute certificate table if it exists
         cert_table = pe.OPTIONAL_HEADER.DATA_DIRECTORY[pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_SECURITY']]
-        if cert_table.Size:
-            warnings.warn(f'Authenticode signature has been removed from {lib_name}', UserWarning)
-            cert_table.VirtualAddress = 0
-            cert_table.Size = 0
+        cert_table.VirtualAddress = 0
+        cert_table.Size = 0
 
         # all changes to headers are done; serialize the PE file
         lib_data = pe.write()
